{*******************************************************************}
{                                                                   }
{  WPrpDlg.pas, Copyright (c) Simon Page 1998                       }
{                                                                   }
{  Version....: 1.13                                                }
{                                                                   }
{  Purpose....: Property editor for TWheelMouse component           }
{                                                                   }
{  Requires...: Delphi 2 or greater                                 }
{                                                                   }
{*******************************************************************}

{$I WheelMs.inc}

unit WhPrpDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
     Buttons, ExtCtrls, WheelMs, DsgnIntf;

type
  TWhPropDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    SrcList: TListBox;
    DstList: TListBox;
    SrcLabel: TLabel;
    DstLabel: TLabel;
    IncludeBtn: TSpeedButton;
    IncAllBtn: TSpeedButton;
    ExcludeBtn: TSpeedButton;
    ExAllBtn: TSpeedButton;
    AllSelBtn: TButton;
    MsgOptGBox: TGroupBox;
    SendWMRad: TRadioButton;
    SendSimRad: TRadioButton;
    MsgTypeGBox: TGroupBox;
    VHScrollRad: TRadioButton;
    VScrollRad: TRadioButton;
    HScrollRad: TRadioButton;
    HCursorRad: TRadioButton;
    VCursorRad: TRadioButton;
    Label1: TLabel;
    procedure AllSelBtnClick(Sender: TObject);
    procedure DstListClick(Sender: TObject);
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExcAllBtnClick(Sender: TObject);
    procedure AddSelected(List: TCustomListBox; Items: TStrings);
    procedure RemoveSelected(List: TCustomListBox; Items: TStrings);
    procedure SetItem(List: TListBox; Index: Integer);
    function  GetFirstSelection(List: TCustomListBox): Integer;
    procedure SetButtons;
    procedure UpdateOptionCtrls;
    procedure VHScrollRadClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SendWMRadClick(Sender: TObject);
  private
    { Private declarations }
    FLocRec   : TWheelCtrlRec;
    FTempCtrls: TWheelCtrlList;
    procedure ReadSelectedRecord;
    procedure UpdateMultiSelect;
    procedure UpdateSelectedRecord;
  public
    { Public declarations }
  end;

  TWheelCtrlPropEd = class(TPropertyEditor)
  public
    procedure Edit; override;
    function  GetAttributes: TPropertyAttributes; override;
    function  GetValue: String; override;
  end;

  TWheelMouseCompEd = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function  GetVerb(Index: Integer): string; override;
    function  GetVerbCount: Integer; override;
  end;

{$IFDEF VERSION2_DOWN}
  function EditCtrlListProp(Form: TForm; CtrlList: TWheelCtrlList): Boolean;
{$ELSE}
  function EditCtrlListProp(Form: TCustomForm; CtrlList: TWheelCtrlList): Boolean;
{$ENDIF}

var
  WhPropDlg: TWhPropDlg;

implementation

{$R *.DFM}

const
  { Default options used for new TWheelCtrl items }
  DefMsgType = mtVCursor;
  DefSendOpt = soSendSim;

{$IFDEF VERSION2_DOWN}
function EditCtrlListProp(Form: TForm; CtrlList: TWheelCtrlList): Boolean;
{$ELSE}
function EditCtrlListProp(Form: TCustomForm; CtrlList: TWheelCtrlList): Boolean;
{$ENDIF}
var
  I: Integer;
begin
  Result := False;
  with TWhPropDlg.Create(Application) do
    try
      SrcList.Items.Clear;
      DstList.Items.Clear;
      { The source list stores the controls names and references, the dest.
        list stores the control names and a reference to a TWheelCtrl item }
      FTempCtrls.Assign(CtrlList);
      for I := 0 to FTempCtrls.Count - 1 do
        DstList.Items.AddObject(FTempCtrls[I].Control.Name, FTempCtrls[I].Control);
      with Form do
        for I := 0 to ComponentCount - 1 do
          if (Components[I] is TWinControl)
             and (DstList.Items.IndexOf(Components[I].Name) = -1) then
                SrcList.Items.AddObject(Components[I].Name, Components[I]);
      SetButtons;
      if ShowModal = mrOK then
      begin
        Result := not CtrlList.Equals(FTempCtrls);
        if Result then CtrlList.Assign(FTempCtrls)
      end;
    finally
      Free;
    end;
end;


{ TWhPropDlg ********************************************************}

procedure TWhPropDlg.IncludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(SrcList);
  AddSelected(SrcList, DstList.Items);
  SetItem(SrcList, Index);
end;

procedure TWhPropDlg.ExcludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(DstList);
  RemoveSelected(DstList, SrcList.Items);
  SetItem(DstList, Index);
  ReadSelectedRecord;
end;

procedure TWhPropDlg.IncAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to SrcList.Items.Count - 1 do
  begin
    { copy the controls name and reference to the dest. list }
    DstList.Items.AddObject(SrcList.Items[I], SrcList.Items.Objects[I]);
    FTempCtrls.Add(TWinControl(SrcList.Items.Objects[I]), DefMsgType, DefSendOpt);
  end;
  SrcList.Items.Clear;
  SetItem(SrcList, 0);
end;

procedure TWhPropDlg.ExcAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to DstList.Items.Count - 1 do
  begin
    { copy the control name and reference back to source list }
    SrcList.Items.AddObject(DstList.Items[I], DstList.Items.Objects[I]);
    { Delete the CtrlRec which is no longer needed }
    FTempCtrls.Delete(FTempCtrls.IndexOf(TWinControl(DstList.Items.Objects[I])));
  end;
  DstList.Items.Clear;
  SetItem(DstList, 0);
end;

procedure TWhPropDlg.AddSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
    begin
    { copy the controls name and reference to the dest. list }
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
      FTempCtrls.Add(TWinControl(List.Items.Objects[I]), DefMsgType, DefSendOpt);
      List.Items.Delete(I);
    end;
end;

procedure TWhPropDlg.RemoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
    begin
     { copy the control name and reference  back to source list }
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
     { Free the TWheelCtrl item which is no longer needed }
      FTempCtrls.Delete(FTempCtrls.IndexOf(TWinControl(List.Items.Objects[I])));
      List.Items.Delete(I);
    end;
end;

procedure TWhPropDlg.SetButtons;
var
  SrcEmpty, DstEmpty: Boolean;
begin
  UpdateMultiSelect;
  SrcEmpty := SrcList.Items.Count = 0;
  DstEmpty := DstList.Items.Count = 0;
  IncludeBtn.Enabled := not SrcEmpty;
  IncAllBtn.Enabled := not SrcEmpty;
  ExcludeBtn.Enabled := not DstEmpty;
  ExAllBtn.Enabled := not DstEmpty;
  UpdateOptionCtrls;
end;

function TWhPropDlg.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

procedure TWhPropDlg.SetItem(List: TListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do
  begin
    SetFocus;
    MaxIndex := List.Items.Count - 1;
    if Index = LB_ERR then Index := 0
    else if Index > MaxIndex then Index := MaxIndex;
    Selected[Index] := True;
  end;
  SetButtons;
end;

procedure TWhPropDlg.UpdateMultiSelect;
begin
  AllSelBtn.Enabled := DstList.SelCount > 1;
end;

procedure TWhPropDlg.UpdateOptionCtrls;
var
  Enable: Boolean;
  I: Integer;
  Tag: Integer;
begin
  Enable := DstList.SelCount > 0;
  for I := 0 to ComponentCount - 1 do
  begin
    Tag := Components[I].Tag;
    if (Tag > 0) and (Tag < 100) then
      TControl(Components[I]).Enabled := Enable
    else
      if Tag >= 100 then
        TControl(Components[I]).Enabled := SendSimRad.Checked and Enable;
  end;
end;

procedure TWhPropDlg.DstListClick(Sender: TObject);
begin
  UpdateMultiSelect;
  ReadSelectedRecord;
  UpdateOptionCtrls;
end;

procedure TWhPropDlg.VHScrollRadClick(Sender: TObject);
begin
  with Sender as TRadioButton do
    FLocRec.MsgType := TMsgTypeOption(Tag - 100);
  UpdateSelectedRecord;
end;

procedure TWhPropDlg.UpdateSelectedRecord;
var
  CtrlRec: TWheelCtrlRec;
begin
  if AllSelBtn.Enabled then Exit;
  if DstList.Selected[DstList.ItemIndex] then
  begin
    CtrlRec.MsgType := FLocRec.MsgType;
    CtrlRec.SendOpt := FLocRec.SendOpt;
    with FTempCtrls do
     CtrlRecs[IndexOf(TWinControl(DstList.Items.Objects[DstList.ItemIndex]))] := CtrlRec;
  end;
end;

procedure TWhPropDlg.ReadSelectedRecord;
begin
  { if we have a multiple selection already don't keep updating for last shown }
  if AllSelBtn.Enabled then Exit;
  if DstList.SelCount = 0 then Exit;
  if DstList.Selected[DstList.ItemIndex] then
    FLocRec := FTempCtrls.CtrlRecs[
      FTempCtrls.IndexOf(TWinControl(DstList.Items.Objects[DstList.ItemIndex]))];
  if FLocRec.SendOpt = soSendWM then SendWMRad.Checked := True
  else SendSimRad.Checked := True;
  case FLocRec.MsgType of
    mtVScroll: VScrollRad.Checked := True;
    mtHScroll: HScrollRad.Checked := True;
    mtVHScroll: VHScrollRad.Checked := True;
    mtVCursor:  VCursorRad.Checked := True;
    mtHCursor:  HCursorRad.Checked := True;
  end;
end;

procedure TWhPropDlg.AllSelBtnClick(Sender: TObject);
var
  I: Integer;
  CtrlRec: TWheelCtrlRec;
begin
  for I := 0 to DstList.Items.Count - 1 do
    if DstList.Selected[I] then
    begin
      CtrlRec.MsgType := FLocRec.MsgType;
      CtrlRec.SendOpt := FLocRec.SendOpt;
      with FTempCtrls do
       CtrlRecs[IndexOf(TWinControl(DstList.Items.Objects[I]))] := CtrlRec;
    end;
end;

procedure TWhPropDlg.FormCreate(Sender: TObject);
begin
  FTempCtrls := TWheelCtrlList.Create(nil);
end;

procedure TWhPropDlg.FormDestroy(Sender: TObject);
begin
  FTempCtrls.Free;
end;

procedure TWhPropDlg.SendWMRadClick(Sender: TObject);
begin
  if SendWMRad.Checked then FLocRec.SendOpt := soSendWM
  else FLocRec.SendOpt := soSendSim;
  UpdateSelectedRecord;
  UpdateOptionCtrls;
end;

{ TWheelCtrlPropEd **************************************************}

function TWheelCtrlPropEd.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

function TWheelCtrlPropEd.GetValue: String;
begin
  Result := Format('(%s)', [GetPropType^.Name]);
end;

procedure TWheelCtrlPropEd.Edit;
begin
  if EditCtrlListProp(Designer.Form, TWheelCtrlList(GetOrdValue)) then
    Modified;
end;
             
{ TWheelCtrlCompEd **************************************************}

procedure TWheelMouseCompEd.ExecuteVerb(Index: Integer);
begin
  if EditCtrlListProp(Designer.Form, TWheelMouse(Component).SpecialCtrls) then
    Designer.Modified
end;

function TWheelMouseCompEd.GetVerb(Index: Integer): string;
begin
  if Index = 0 then Result := '&Special control handling...'
  else Result := '';
end;

function TWheelMouseCompEd.GetVerbCount: Integer;
begin
  Result := 1;
end;


end.

